#include <windows.h>
#include <d3d9.h>
#include <d3dx9.h>

// globals
IDirect3D9* pD3D9 = NULL;
IDirect3DDevice9* pD3DDevice9 = NULL;

// function declarations //
HRESULT InitD3D(HWND hwnd, int width, int height, bool fullscreen);
void Render(void);
void DestroyD3D(void);
LRESULT CALLBACK WindowProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	WNDCLASSEX windowClass;		// window class
	HWND hWND = NULL;

	// fill out the window class structure
	windowClass.cbSize		= sizeof(WNDCLASSEX);
	windowClass.style			= CS_HREDRAW | CS_VREDRAW;
	windowClass.lpfnWndProc		= WindowProc;
	windowClass.cbClsExtra		= 0;
	windowClass.cbWndExtra		= 0;
	windowClass.hInstance		= hInstance;
	windowClass.hIcon		= LoadIcon(NULL, IDI_APPLICATION);			
	windowClass.hCursor		= LoadCursor(NULL, IDC_ARROW);	
	windowClass.hbrBackground	= (HBRUSH)GetStockObject(BLACK_BRUSH);
	windowClass.lpszMenuName	= NULL;						
	windowClass.lpszClassName	= "window";
	windowClass.hIconSm		= LoadIcon(NULL, IDI_WINLOGO);	

	// register the windows class
	if (!RegisterClassEx(&windowClass))
		return 0;
		

	hWND = CreateWindowEx(NULL, "window", "Simple Direct3D Program",
	WS_OVERLAPPEDWINDOW | WS_VISIBLE,
		0, 0, 400, 400,
 		NULL, NULL, hInstance,	 NULL);	

	// check if window creation failed (hwnd would equal NULL)
	if (!hWND)
		return 0;
		
	if (FAILED(InitD3D(hWND, 800, 600, false)))
	{
			DestroyD3D();
			return 0;
		}
		
	MSG	msg;	
			
	while (1)
	{
		PeekMessage(&msg, hWND, NULL, NULL, PM_REMOVE);

		if (msg.message == WM_QUIT)
			break;
		else
		{
			Render();
			
			TranslateMessage(&msg);									DispatchMessage(&msg);
		}				
	}

	DestroyD3D();
	UnregisterClass("window", hInstance);
	return (msg.wParam);
}

HRESULT InitD3D(HWND hwnd, int width, int height, bool fullscreen)
{
	// create the IDirect3D9 object
	pD3D9 = Direct3DCreate9(D3D_SDK_VERSION);
	if (pD3D9 == NULL)
		return E_FAIL;
	
	// get the display mode
	D3DDISPLAYMODE d3ddm;
	pD3D9->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm);

	// set the presentation parameters
	D3DPRESENT_PARAMETERS d3dpp;
	ZeroMemory(&d3dpp, sizeof(d3dpp));
	d3dpp.BackBufferWidth = width;
	d3dpp.BackBufferHeight = height;
	d3dpp.BackBufferCount = 1;
	d3dpp.BackBufferFormat = d3ddm.Format;
	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	d3dpp.Windowed = !fullscreen;
	d3dpp.EnableAutoDepthStencil = true;
	d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
	d3dpp.FullScreen_RefreshRateInHz = D3DPRESENT_RATE_DEFAULT;
	d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;

	// create the device
	if (FAILED(pD3D9->CreateDevice(D3DADAPTER_DEFAULT, 
		D3DDEVTYPE_HAL, hwnd,
		D3DCREATE_SOFTWARE_VERTEXPROCESSING,
		&d3dpp, &pD3DDevice9)))
	{
		return E_FAIL;
	}

	return S_OK;
}

void Render(void)
{
	// clear the back buffer
	pD3DDevice9->Clear(0, NULL, D3DCLEAR_TARGET,
		D3DCOLOR_XRGB(0, 0, 255), 1.0f, 0);

	// begin rendering
	pD3DDevice9->BeginScene();

	// all of your rendering would go here

	// end rendering
	pD3DDevice9->EndScene();
	
	// present the back buffer to the screen
	pD3DDevice9->Present(NULL, NULL, NULL, NULL);
}

void DestroyD3D(void)
{
	if (pD3DDevice9)
	{
		pD3DDevice9->Release();
		pD3DDevice9 = NULL;
	}

	if (pD3D9)
	{
		pD3D9->Release();
		pD3D9 = NULL;
	}
}

LRESULT CALLBACK WindowProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{
		case WM_CREATE:
			return 0;
			break;

		case WM_CLOSE:					// windows is closing
			PostQuitMessage(0);
			return 0;
			break;

		case WM_DESTROY:
			PostQuitMessage(0);
			return 0;
			break;
		
		default:
			break;
	}

	return (DefWindowProc(hwnd, message, wParam, lParam));
}